classdef spidev < handle
  properties
   chipselectpin = ""
   arduinoobj = []
   pins = []
   mode = 0;
   bitrate = 4000000;
   bitorder = 'msbfirst';
  endproperties

  methods
    %function p = spidev(ar, cspin)
    function p = spidev(varargin)
      ARDUINO_SPI_CONFIG = 11;

      if nargin < 2
	error("expects arduino object and chipselect pin");
      endif

      ar = varargin{1}
      cspin = varargin{2}

      

	if mod(nargin, 2) != 0
          error ("arduino: expected property name, value pairs");
        endif
        if !iscellstr (varargin(3:2:nargin))
          error ("arduino: expected property names to be strings");
        endif

	%mode = 0; % 0-3
	%order = 'msbfirst'; %('msbfirst' or 'lsbfirst')
	%bitrate = 4000000;

	for i = 3:2:nargin
          propname = tolower(varargin{i});
          propvalue = varargin{i+1};

	  printf("%s = %s\n", propname, propvalue);
	  if propname == "bitrate"
	    if !isnumeric(propvalue)
              error("bitrate should be a number")
	    endif
	    p.bitrate = propvalue;
          elseif propname == "mode"
	    if !isnumeric(propvalue) || propvalue < 0 || propvalue > 3
              error("mode should be a number betwwen 0 - 3");
	    endif
	    p.mode = propvalue;
          elseif propname == "bitorder"
	    if !ischar(propvalue)
              error("bitorder should be a string");
	    endif
            p.bitorder = tolower(propvalue);
	    if p.bitorder != "msbfirst" && p.bitorder != "lsbfirst"
	      error("bitorder should be 'msbfirst' or 'lsbfirst'");
	    endif
          endif

        endfor


      if (!isa (ar, "arduino"))
       error("expects arduino object");
      endif

       p.chipselectpin = cspin;
       p.arduinoobj = ar;

       # check if is valid CS pin
       validatePin(ar,cspin,'SPI')

       % TODO on calling setup with the CS pin, returns back the other pins that we then setup as used ?
       % sendCommand
       # there only ever one port ??? with CS able to be completely independant ??
       p.pins = ar.get_pingroup(cspin, "SPI");
       if numel(p.pins) != 4
	  error("expected 4 SPI pins but only have %d", numel(p.pins) )
       endif

       # TODO: save old modes and set them via force if we fail trying to alloc the whole group
       try
        for i=1:4
          configurePin(ar, p.pins{i}.name, "spi")
	  if (strcmpi(p.pins{i}.func, "ss"))
            p.chipselectpin = p.pins{i}.name;
	    # overwride mode for digital out
	    configurePinResource(ar, p.pins{i}.name, getResourceOwner(ar, p.pins{i}.name), "digitaloutput", true);
	  endif
        endfor
        # TODO: bitrate
	bitorder = 0;
	if p.bitorder == 'lsbfirst'
          bitorder = 1;
	endif
        [tmp, sz] = sendCommand(p.arduinoobj, "", ARDUINO_SPI_CONFIG, [0 1 p.mode bitorder ]);
       catch
        for i=1:4
          configurePinResource(ar, p.pins{i}.name, p.pins{i}.owner, p.pins{i}.mode, true)
          configurePin(ar, p.pins{i}.name, p.pins{i}.mode)
        endfor
        rethrow (lasterror)
       end_try_catch

    endfunction

    function disp (p)
      printf ("  arduino spidev object with fields of: \n\n");
      printf ("   chipselectpin = %s\n", p.chipselectpin);
      printf ("            mode = %d\n", p.mode);
      printf ("        bitorder = %s\n", p.bitorder);
      printf ("         bitrate = %d\n", p.bitrate);
      printf ("            pins = ");
      for i=1:4
	printf("%s(%s) ", p.pins{i}.name, p.pins{i}.func)
      endfor
     printf("\n");
     % Mode, Bitrate, Bitorder
     
      printf ("\n");
    endfunction

  endmethods
endclassdef
